#include <signal.h>
#include "libmpdclient.h"
#include "interface.h"
#include "radiolog.h"
#include "inifile.h"
#include "serkeyb.h"

// tine textul ce se trimite pe seriala
char line1[STR_LENGTH],line2[STR_LENGTH],line3[STR_LENGTH],line4[STR_LENGTH];
char old_s1[STR_LENGTH],old_s2[STR_LENGTH],old_s3[STR_LENGTH],old_s4[STR_LENGTH]; //ce am trimis ultima data
char stations[100][STR_LENGTH];
char station_urls[100][STR_LENGTH];

//MPD stuff
char mpd_song[255] = "no song";
struct config {
	const char *host, *password;
	const char *format;
	char *buffer;
	long port;
	unsigned columns, background;
} config;
//MPD connection
mpd_Connection *conn;

char commands[500]; // caractere venite pe seriala, si trebuie procesate
int command_ptr=0; // cate caractere sunt in commands[] de procesat (0 = niciunul, 1 = primul, etc)

int btn1=0, btn2=0, encoder=0;

int state=MENU_NONE; // in care meniu suntem (-1=invalid)
int menu_select=0; // care optiune de meniu e aleasa (0=none)
int old_menu_select=0;

int num_stations=0; //numarul de posturi definite
int selected_radio=0; //postul ales

int serport; //file descriptor pentru portul serial
int linux_ver=0; //0-linux normal, 1-openwrt
int no_console=0; //no_console = 1 - nu exista consola (fara getchar, printf, etc)

int interface_type=1; //mod initial
int auto_play=0; //pornire automata radio
int playing=0; //daca canta momentan
int volume=60; // pornim default pe 60, incarcam din fisier
int volume_increment = 5; // cu cat modificam volum
int allow_exit = 0; // nu permite iesirea din program cu cancel (merge ctrl-C si kill sau killall)

char doing = 1;

FILE *config_file; //fisierul cu configuri

void alter(char *str,char *out) {
    strcpy(out, str);

    out[0] = '-';
    out[1] = '>';
    out[14] = '<';
    out[15] = '-';
}

int radio_connect(struct config *config) {
    int res=0, tries=0;
    
    log_debug(7,"radio_connect(): old conn=%d\n\r", conn);
    if (conn)
	mpd_closeConnection(conn);
    
    while (tries < 10) {
	res=0;
	tries++;
	conn = mpd_newConnection(config->host, config->port, 10);
	log_debug(6, "radio_reconnect: MPD: reconnectare - err(%d) - tries = %d\n\r", conn->error, tries);
	if (conn) break;
	res = -1;
    }
    return res;
}


int get_mpd_song(void) { //cantec curent mpd, baga in mpd_song
	mpd_InfoEntity *info;
	mpd_Song *song;
	mpd_Status *status;
	int res=0;

	log_debug(7,"get_mpd_song: 1\n\r");
	//FIXME-AJ: test conn != NULL, si reconnect
	if (!conn)
	    res = radio_connect(&config);
	if (res < 0) {
	    log_debug(0,"get_mpd_song: MPD: connection lost, can't reconnect\n\r");
	}
	mpd_sendStatusCommand(conn);
	log_debug(7,"get_mpd_song: 2 sendStatusCommand - err(%d)\n\r", conn->error);
	if (conn->error) {
	    log_debug(7,"- %s\n\r", conn->errorStr);
	    return -1;
	}
	status = mpd_getStatus(conn);
	log_debug(7,"get_mpd_song: 3 getStatus - err(%d)\n\r", conn->error);
	if (conn->error) {
	    log_debug(7,"- %s\n\r", conn->errorStr);
	    return -1;
	}
	mpd_nextListOkCommand(conn);
	log_debug(7,"get_mpd_song: 4 ListOkCmd - err(%d)\n\r", conn->error);
	if (conn->error) {
	    log_debug(7,"- %s\n\r", conn->errorStr);
	    return -1;
	}

	/* Get song info */
	mpd_sendCurrentSongCommand(conn);      
	log_debug(7,"get_mpd_song: 5 sendCurrentSong - err(%d)\n\r", conn->error);
	if (conn->error) {
	    log_debug(7,"- %s\n\r", conn->errorStr);
	    return -1;
	}
	if (conn->error) return -1;
	info = mpd_getNextInfoEntity(conn);
	if (!info) return -2;
	if (info->type!=MPD_INFO_ENTITY_TYPE_SONG) {
		mpd_freeInfoEntity(info);
		return -3;
	}

	/* Format song string */
	//format_song(config, name, info->info.song);
	song = info->info.song;
	if (song->title != NULL) {
	    strcpy(mpd_song, song->title);
	} else if (song->artist != NULL) {
	    strcpy(mpd_song, song->artist);
	} else if (song->track != NULL) {
	    strcpy(mpd_song, song->track);
	} else {
	    strcpy(mpd_song, "no song title");
	}
	log_debug(4,"MPD: got song title: %s\n\r", mpd_song);
	mpd_freeInfoEntity(info);
	return 1;
}

void radio_stop() {
    int res=0;
    
    if (!conn) 
	res = radio_connect(&config);

    if (res < 0) {
	log_debug(6,"radio_stop(): MPD: could not reconnect\n\r");
    } else {
	mpd_sendStopCommand(conn);
	mpd_finishCommand(conn);
	playing = 0;
	log_debug(6, "do_things(): MPD: trimis stop - err(%d)\n\r", conn->error);
    }
}

void radio_play(int station) {
    int res=0;
    
    if (!conn) 
	res = radio_connect(&config);

    if (res < 0) {
	log_debug(6,"radio_play(): MPD: could not reconnect\n\r");
    } else {
	mpd_sendPlayCommand(conn, station);
	mpd_finishCommand(conn);
	playing = 1;
	log_debug(6, "do_things(): MPD: trimis play - err(%d)\n\r", conn->error);
    }
}

int send_string(char adr, char *str) {
    char s[STR_LENGTH],s1[STR_LENGTH];
    int i, res=1, spaces=0;
    
    strcpy(s1,str);
    if (strlen(str) < 16) {
	spaces = 16 - strlen(str);
	for (i=0;i<spaces;i++)
	    s1[strlen(str) + i] = ' ';
	s1[strlen(str) + i] = '\0';
    }
    if (strlen(str) > 64) {
	s1[64] = '\0';
    }
    // asamblam linia 1 pentru LCD
    sprintf(s,"%c%c%c%s",0xFF,strlen(s1)+1,adr,s1); //modificat de dan, alex

    strcpy(s1,s);
    s1[0]=' ';
    s1[1]=' ';
    s1[2]=' ';
    
    log_debug(1,"SER_TX: %s ",s1);
    log_hex(1,s);

    write(serport,s,strlen(s));
    return 1;
}

/* trimite pe seriala textul din line1..line4*/
int send_screen() {
    char s1[STR_LENGTH],s2[STR_LENGTH],s3[STR_LENGTH],s4[STR_LENGTH];
    int res;
    
    strcpy(s1,line1);
    strcpy(s2,line2);
    strcpy(s3,line3);
    strcpy(s4,line4);

    //punem -> <- de meniu
    if (state == MENU_MAIN) {
        if (menu_select == 1) alter(line1,s1);
	if (menu_select == 2) alter(line2,s2);
	if (menu_select == 3) alter(line3,s3);
	if (menu_select == 4) alter(line4,s4);
    }

    log_debug(0,"1:%s:(%d)\r\n2:%s:(%d)\r\n3:%s:(%d)\r\n4:%s:(%d)\r\n",s1,strlen(s1),s2,strlen(s2),s3,strlen(s3),s4,strlen(s4));

    if (strcmp(s1,old_s1)) { //daca nu e 0, inseamna ca-s diferite
	res = send_string(0x01,s1);
	if (res < 0) return res;
	strcpy(old_s1,s1); // salvam ce-am trimis
    }
    if (strcmp(s2,old_s2)) { //daca nu e 0, inseamna ca-s diferite
	res = send_string(0x02,s2);
	if (res < 0) return res;
	strcpy(old_s2,s2); // salvam ce-am trimis
    }
    if (strcmp(s3,old_s3)) { //daca nu e 0, inseamna ca-s diferite
	res = send_string(0x03,s3);
	if (res < 0) return res;
	strcpy(old_s3,s3); // salvam ce-am trimis
    }
    if (strcmp(s4,old_s4)) { //daca nu e 0, inseamna ca-s diferite
	res = send_string(0x04,s4);
	if (res < 0) return res;
	strcpy(old_s4,s4); // salvam ce-am trimis
    }
    return RD_OK;
}

void up_down(data) {
    switch(data) {
        case 's': 
		menu_select++; 
		if (menu_select>4) 
		    menu_select = 1;
	    break;
	case 'w': 
		menu_select--; 
		if (menu_select<1)
		    menu_select = 4;
	    break;
    }
}

void update_display() {
    FILE *inp;
    time_t t;
    struct tm *tmp;
    int l1,l2,l3,l4,i,res;
    char s[STR_LENGTH],s1[STR_LENGTH],s2[STR_LENGTH],s3[STR_LENGTH];
    char *p;

    t = time(NULL);
    tmp = localtime(&t);

    switch (state) {
	case MENU_MAIN:
    	    sprintf(line1,"%s",STRING_MAIN_PLAY);
	    sprintf(line2,"%s",STRING_MAIN_SELECT);
	    sprintf(line3,"%s",STRING_MAIN_STANDBY);
	    sprintf(line4,"%s",STRING_MAIN_WEATHER);
	    break;
	case MENU_PLAY:
	    sprintf(line1,"%s",stations[selected_radio]);
	    
	    res=get_mpd_song();
	    if (res < 0) {
		sprintf(line1, "%s - Err:%d", "Probleme interfata MPD",res);
		printf("EROARE: %s\n\r",line1);
		strcpy(mpd_song, MPD_NO_SONG);
	    }
	    sprintf(line2,"%s",mpd_song);
	    //sprintf(line3,"%s",station_urls[selected_radio]); //daca vrem URL
	    sprintf(line3,"Volum: %d", volume);
	    strftime(line4, STR_LENGTH, "%d.%m.%Y %H:%M", tmp);
	    break;
	case MENU_SELECTION:
	    sprintf(line1, "> %s", stations[menu_select%num_stations]);
	    sprintf(line2, "%s", stations[(menu_select+1)%num_stations]);
	    sprintf(line3, "%s", stations[(menu_select+2)%num_stations]);
	    sprintf(line4, "%s", stations[(menu_select+3)%num_stations]);
	    break;
	case MENU_WEATHER:
//	    sprintf(s, "wget http://juve.ro/weather.php?city=%s -O weather.forecast",iniFind(config_file,"WEATHER_CITY","MAIN"));
//	    system(s); //in caz ca vrem sa executam noi, nu din cron
	    inp = fopen("/usr/share/radio/weather.forecast","r");
	    if (inp == NULL) {
		sprintf(line1, "%s", "OOOOPS!!!");
		sprintf(line2, "%s", "Error reading");
		sprintf(line3, "%s", "weather.forecast");
		break;
	    }
	    fgets(s,STR_LENGTH,inp); //prima linie conditii curente
	    fgets(s,STR_LENGTH,inp); 
	    if (s[strlen(s)-1] = '\n') s[strlen(s)-1] = '\0';
	    sprintf(line1, "%s", s);
	    fgets(s,STR_LENGTH,inp); 
	    if (s[strlen(s)-1] = '\n') s[strlen(s)-1] = '\0';
	    sprintf(line2, "%s", s);
	    fgets(s,STR_LENGTH,inp); 
	    if (s[strlen(s)-1] = '\n') s[strlen(s)-1] = '\0';
	    sprintf(line3, "%s", s);
	    fgets(s,STR_LENGTH,inp); 
	    if (s[strlen(s)-1] = '\n') s[strlen(s)-1] = '\0';
	    sprintf(line4, "%s", s);
	    fclose(inp);
	    break;
	case MENU_STANDBY:
	    inp = fopen("/usr/share/radio/weather.forecast","r");
	    if (inp == NULL) {
		sprintf(line1, "%s", "OOOOPS!!!");
		sprintf(line2, "%s", "Error reading");
		sprintf(line3, "%s", "weather.forecast");
		break;
	    }
	    fgets(s,40,inp); 
	    fclose(inp);
	    p = strchr(s,'\n'); *(p) = '\0'; // stergem restul de string de la \n incolo (adica forecast)
	    //s[0] = ' '; // punem spatiu in loc de zi
    	    sprintf(line1,"%s",STRING_STANDBY_1);
	    sprintf(line2,"%s",STRING_STANDBY_2);
	    strftime(line3, STR_LENGTH, "%d.%m.%y ", tmp);
	    strcat(line3,s); // adaugam temperaturile de azi, la data
	    strftime(line4, STR_LENGTH, "    %H:%M:%S    ", tmp);
	    break;
    }
}

int do_things(char data) {
    int doing = 1, oms, cnt=0;
    const char *ini_str;
	//schimbari in functie de intrari
	switch (state) {
	    case MENU_MAIN: 
		up_down(data);
		
		if (data == BTN_OK) { //intram in submeniu
		    oms = menu_select;
		    menu_select = 0;
		    switch (oms) {
			case 1:
			    state = MENU_PLAY; 
			    if (!playing) {
				radio_play(selected_radio);
			    }
			    strcpy(mpd_song, MPD_NO_SONG);
			    break;
			case 2:state = MENU_SELECTION; menu_select = selected_radio; break;
			case 3:state = MENU_STANDBY; break;
			case 4:state = MENU_WEATHER; break;
		    }
		}
		if (data == BTN_CANCEL)
		    if (allow_exit) {
			doing = 0;
			log_debug(8, "do_things(): requested exit\n\r");
		    }				    
		break;
	    case MENU_PLAY: 
		if (data == BTN_UP) {
		    volume -= volume_increment;
		    if (volume < 0) volume = 0;
		    mpd_sendSetvolCommand(conn, volume);
		    mpd_finishCommand(conn);
		    log_debug(6, "do_things(): MPD: trimis volum(%d) - err(%d)\n\r", volume, conn->error);	    
		}
		if (data == BTN_DOWN) {
		    volume += volume_increment;
		    if (volume > 100) volume = 100;
		    mpd_sendSetvolCommand(conn, volume);
		    mpd_finishCommand(conn);		    
		    log_debug(6, "do_things(): MPD: trimis volum(%d) - err(%d)\n\r", volume, conn->error);	    
		}
		if (data == BTN_OK) {
		    menu_select = 1;
		    state = MENU_MAIN;
		    update_display();
		}
		if (data == BTN_CANCEL) {
		    menu_select = 1;
		    state = MENU_MAIN;
		    update_display();
		    radio_stop();
		}
		break;
	    case MENU_SELECTION: 
		if (data == BTN_DOWN) {
		    menu_select++;
		    if (menu_select >= num_stations) menu_select=0;
		    log_debug(6,"do_things: menu_select = %d\n\r", menu_select);
		    update_display();
		    if (interface_type == 3) {
			radio_play(menu_select);
		    }
		}
		if (data == BTN_UP) {
		    menu_select--;
		    if (menu_select < 0) menu_select=num_stations-1; //de la 0 la num_stations-1
		    log_debug(6,"do_things: menu_select = %d\n\r", menu_select);
		    update_display();
		    if (interface_type == 3) {
			radio_play(menu_select);
		    }
		}
		if (data == BTN_OK) {
		    selected_radio = menu_select % num_stations; // tinem minte radio selectat
		    //salvam in fisieru de config
		    ini_str = iniFind(config_file,"ACTIVE_STATION","RADIO");
		    if (ini_str != NULL) {
			cnt = 2+strlen("ACTIVE_STATION =")+strlen(ini_str);
			log_debug(8,"do_things(): save ACTIVE_STATION: seek de %d, len(inistr)=%d\n\r",cnt,strlen(ini_str));
			fseek(config_file, -cnt, SEEK_CUR);
			fprintf(config_file,"ACTIVE_STATION = %d\n",selected_radio);
			fflush(config_file);
		    }
		    menu_select = 1;
		    state = MENU_PLAY;
		    update_display();
		}
		if (data == BTN_CANCEL) {
		    menu_select = 2;
		    radio_stop();
		    state = MENU_MAIN;
		    update_display();
		}
		break;
	    case MENU_STANDBY: 
		if (data == BTN_OK) {
		    menu_select = 1;
		    state = MENU_MAIN;
		    update_display();
		}
		if (data == BTN_CANCEL) {
		    menu_select = 3;
		    state = MENU_MAIN;
		    update_display();
		}
		break;
	    case MENU_WEATHER: 
		if (data == BTN_CANCEL) {
		    menu_select = 4;
		    state = MENU_MAIN;
		    update_display();
		}
		if (data == BTN_OK) {
		    menu_select = 1;
		    state = MENU_MAIN;
		    update_display();
		}
		break;
	}
    return doing;
}

void main_loop(void) {
    char data;
    int count=0;
    int crt_ptr=0;
    
    while (doing) { //!conditie de iesire? sau doar ctrl-c
	//verificam intrari
	usleep(70000);
	data = check_inputs();
	if (count > 200) {
	    count = 0; 
	    update_display();
	    send_screen();
	}

	if (state == MENU_STANDBY) count+=20; // daca suntem in meniul de STANDBY vrem autorefresh (pentru ceas)
	if (state == MENU_PLAY) count++; // daca suntem in meniul de PLAY vrem autorefresh (pentru play) dar mai incet

	if (data <= 0) continue; //no data, sleep (trebuie sa verific. si timp aici)

	while (crt_ptr < command_ptr) {
	    log_debug(6,"main_loop: got a char (commands[%d<%d]=%c)\n\r",crt_ptr,command_ptr,commands[crt_ptr]);
	    doing = do_things(commands[crt_ptr++]);
	    count = 201;
	}
	crt_ptr = 0;
	command_ptr = 0;
    }
}

int init(struct config *config) {
    FILE *inp;
    state = MENU_MAIN;
    menu_select = 1;
    char s[STR_LENGTH];
    const char *inistring;
    int len, ok=6, i, res;
    struct termios options; // setari de port serial
    
    
    config_file = fopen("radio.conf", "r+");
    if (config_file == NULL) //incercam si config instalat
	config_file = fopen("/etc/radio.conf", "r+");
    if (config_file == NULL) {
	printf("Eroare la citire fisier config: radio.conf sau /etc/radio.conf\n\r");
	return -1;
    }

    //logfile
    inistring = iniFind(config_file, "LOG_FILE", "MAIN");
    res = log_open(inistring);
    // daca nu am reusit sa deschidem, incercam la stdout
    if (res < 0) {
	res = log_open("stdout");
    }
    // daca nici acolo, abandonam
    if (res < 0) return res;
    
    inistring = iniFind(config_file, "DEBUG", "MAIN");
    res = sscanf(inistring,"%d",&i);
    if (res < 0) {
	log_set_level(LOG_MAX_LEVEL); //max
	log_debug(7,"INIT: problema la citire [MAIN]DEBUG\n\r");
    } else {
	log_set_level(i);
	log_debug(3,"INIT: [MAIN]DEBUG=%d\n\r",i);
    }

    inistring = iniFind(config_file, "NO_CONSOLE", "MAIN");
    if (inistring != NULL) {
	res = sscanf(inistring,"%d",&no_console);
	if (res < 0) {
	    no_console = 0; // ceva problema, lasam consola
	}
	if (no_console) {
	    if (log_get_fp() == stdout) {
		log_set_level(0); // daca tiparim la stdout, si e definit no_console, atunci punem nivel 0
	    }
	}	    
    }

    inistring = iniFind(config_file, "TYPE", "INTERFACE");
    if (inistring != NULL) {
	res = sscanf(inistring,"%d",&interface_type);
	if (res < 0) {
	    interface_type = 1; // ceva problema, lasam default
	}
	log_debug(3,"INIT: [INTERFACE]TYPE=%d\n\r", interface_type);
    }

    inistring = iniFind(config_file, "VOLUME", "INTERFACE");
    if (inistring != NULL) {
	res = sscanf(inistring,"%d",&volume);
	if (res < 0) {
	    volume = 60; // ceva problema, lasam default
	}
	log_debug(3,"INIT: [INTERFACE]VOLUME=%d\n\r", volume);
    }

    inistring = iniFind(config_file, "VOLUME_INCREMENT", "INTERFACE");
    if (inistring != NULL) {
	res = sscanf(inistring,"%d",&volume_increment);
	if (res < 0) {
	    volume_increment = 5; // ceva problema, lasam default
	}
	log_debug(3,"INIT: [INTERFACE]VOLUME_INCREMENT=%d\n\r", volume_increment);
    }

    inistring = iniFind(config_file, "AUTO_PLAY", "INTERFACE");
    if (inistring != NULL) {
	res = sscanf(inistring,"%d",&auto_play);
	if (res < 0) {
	    auto_play = 0; // ceva problema, lasam default
	}
	log_debug(3,"INIT: [INTERFACE]AUTO_PLAY=%d\n\r", auto_play);
	if (auto_play)
	    state = MENU_PLAY;
    }

    inistring = iniFind(config_file, "ALLOW_EXIT", "INTERFACE");
    if (inistring != NULL) {
	res = sscanf(inistring,"%d",&allow_exit);
	if (res < 0) {
	    allow_exit = 0; // ceva problema, lasam default
	}
	log_debug(3,"INIT: [INTERFACE]ALLOW_EXIT=%d\n\r", allow_exit);
    }

    inistring = iniFind(config_file, "OPENWRT", "MAIN");
    if (inistring != NULL) {
	res = sscanf(inistring,"%d",&linux_ver);
    } else linux_ver = 0;
    if (res < 0) {
	linux_ver = 0; // ceva problema, punem debug pe maxim
    } 
    log_debug(3,"INIT: [MAIN]LINUX_VER=%d\n\r", linux_ver);

    inistring = iniFind(config_file, "HOST", "MPD");
    config->host = (char *) malloc(50);
    if (inistring != NULL) {
	strcpy((char *)config->host,inistring);
    } else {
	config->host = DEFAULT_HOST;
    }
    log_debug(3,"INIT: MPD HOST: %s\n\r", config->host);

    inistring = iniFind(config_file, "PORT", "MPD");
    if (inistring != NULL) {
	sscanf(inistring,"%d",&(config->port));
    } else {
	config->port = DEFAULT_PORT;
    }
    log_debug(3,"INIT: MPD PORT: %d \r\n",config->port);

    //connecting to MPD:
    conn = mpd_newConnection(config->host, config->port, 10);
    
    if (conn->error) {
	log_debug(0,"could not connect to MPD (%s:%ld)\n\r", config->host, config->port);
	log_debug(0,"error: %d = %s\n\r", conn->error, conn->errorStr);
	return -1;
    }
    
    log_debug(4,"INIT: MPD: connected to MPD %d\n\r",conn->error);
    log_debug(4,"INIT: MPD: sending clear command\n\r");

    mpd_sendCommandListBegin(conn);
    mpd_sendClearCommand(conn);
    log_debug(4,"INIT: MPD: Err?: %d-%s\n\r", conn->error, conn->errorStr);

    inistring = iniFind(config_file, "NUM_STATIONS", "MAIN");
    if (inistring != NULL) {
	sscanf(inistring,"%d",&num_stations);
    }
    log_debug(3,"INIT: expected %d stations\r\n",num_stations);
    
    for (i=0; i < num_stations; i++) {
	sprintf(s,"STATION_%d",i);
	inistring = iniFind(config_file, "NAME", s);
	if (inistring != NULL) {
	    log_debug(3,"INIT: station name: %s\n\r", inistring);
	    strcpy(stations[i],inistring);
	}
	inistring = iniFind(config_file, "URL", s);
	if (inistring != NULL) {
	    log_debug(3,"INIT: station url: %s\n\r", inistring);
	    strcpy(station_urls[i],inistring);
	    
	    mpd_sendAddCommand(conn, station_urls[i]);
	    log_debug(3,"INIT: sent to MPD: %s Err?: %d-%s\n\r", inistring, conn->error, conn->errorStr);
	}
    }

    mpd_sendCommandListEnd(conn);
    mpd_finishCommand(conn);
    log_debug(3,"INIT: gatat cu lista la MPD - err(%d)\n\r",conn->error);

    //setam volum initial
    mpd_sendSetvolCommand(conn, volume);
    mpd_finishCommand(conn);
    log_debug(3,"INIT: volum trimis la MPD - err(%d)\n\r",conn->error);
    
    inistring = iniFind(config_file, "ACTIVE_STATION", "RADIO");
    if (inistring != NULL) {
	sscanf(inistring,"%d",&selected_radio);
    }
    log_debug(3,"INIT: selected radio = (%d) - %s\r\n",selected_radio,stations[selected_radio]);
    
    if (auto_play) {
	log_debug(3,"INIT: auto_play - station %d\n\r",selected_radio);

	mpd_sendPlayCommand(conn, selected_radio);
	mpd_finishCommand(conn);		    
	log_debug(3,"INIT: MPD: trimis play - err(%d)\n\r",conn->error);
    }
    
    //cautam portul de seriala
    serport = open(iniFind(config_file,"SERIAL","MAIN"), O_RDWR | O_NDELAY);
    if (serport > 0) {
	log_debug(3,"INIT: deschis portul serial %s\n\r", iniFind(config_file,"SERIAL","MAIN"));
    } else {
	log_debug(0,"INIT:EROARE: probleme la deschidere port serial %s - %d\n\r", iniFind(config_file,"SERIAL","MAIN"),serport);
    }
    
    fcntl(serport, F_SETFL, FNDELAY); // read returneaza 0 daca nu sunt caractere
    
    tcgetattr(serport, &options);
    cfsetispeed(&options, B115200);
    cfsetospeed(&options, B115200);
    
    options.c_cflag |= (CLOCAL | CREAD);
    
    tcsetattr(serport, TCSANOW, &options);

    if (linux_ver != 1)
	changemode(1); //initializam mod 0 pentru getch()
    
    sprintf(s,"%c%c%c%c%c%c%c\0",0xFF,5,0x01,SYNC1,SYNC2,SYNC3,SYNC4);
    write(serport,s,strlen(s));
    
    return RD_OK;
}

void shutdown(void) {

    log_debug(6,"SHUTDOWN: called\n\r");

    fclose(config_file);

    close(serport);
    if (linux_ver != 1)
	changemode(0);
    log_close();
}

static void radio_quit(int sig) {
    //shutdown();
    
    doing = 0;
    
    signal(sig, radio_quit);
}

static void radio_pipe(int sig) {
    radio_connect(&config);
    
    signal(sig, radio_pipe);
}

int main(void) {
    int res=0, oldstate;


    //trap ^C
    signal(SIGINT, radio_quit);
    //trap SIGTERM - kill/killall
    signal(SIGTERM, radio_quit);
    
    signal(SIGPIPE, radio_pipe);

    state = MENU_MAIN;
    //initializare
    res = init(&config);
    if (res < 0) return res;

    oldstate = state;
    state=MENU_NONE;
    sprintf(line1,"%s", "----------------");
    sprintf(line2,"%s", "starting ./radio");
    sprintf(line3,"%s", "version: 1.50-09");
    sprintf(line4,"%s", "----------------");
    send_screen();
    
    sleep(1);

    //trimitem primul meniu
    state = oldstate; //MENU_MAIN in cazul in care nu a fost modificat in init()
    update_display();
    send_screen();

    //dothings
    main_loop();
    
    shutdown();
    return RD_OK;
}
